@@ -731,15 +731,14 @@ int SolveMultipleFunction::calculate(MathStructure &mstruct, const MathStructure
 		EvaluationOptions eo2 = eo;
 		eo2.isolate_var = &vargs[1][i];
 		for(size_t i2 = 0; i2 < i; i2++) {
-			size_t index = eorder[i2];
-			if(mstruct[index].isVector()) {
+			if(mstruct[i2].isVector()) {
 				msolve.transform(STRUCT_LOGICAL_OR);
-				for(size_t i4 = 1; i4 < mstruct[index].size(); i4++) msolve.addChild(msolve[0]);
-				for(size_t i4 = 0; i4 < mstruct[index].size(); i4++) {
-					msolve[i4].replace(vargs[1][i2], mstruct[index][i4]);
+				for(size_t i4 = 1; i4 < mstruct[i2].size(); i4++) msolve.addChild(msolve[0]);
+				for(size_t i4 = 0; i4 < mstruct[i2].size(); i4++) {
+					msolve[i4].replace(vargs[1][i2], mstruct[i2][i4]);
 				}
 			} else {
-				msolve.replace(vargs[1][i2], mstruct[index]);
+				msolve.replace(vargs[1][i2], mstruct[i2]);
 			}
 		}
 		msolve.eval(eo2);
@@ -830,7 +829,7 @@ int SolveMultipleFunction::calculate(MathStructure &mstruct, const MathStructure
 		for(size_t i2 = 0; i2 < i; i2++) {
 			for(size_t i3 = 0; i3 <= i; i3++) {
 				if(i2 != i3) {
-					size_t index1 = eorder[i2];
+					size_t index1 = i2;
 					size_t index2 = i3;
 					if(mstruct[index2].isVector()) {
 						MathStructure m;
@@ -1273,7 +1272,7 @@ int NewtonRaphsonFunction::calculate(MathStructure &mstruct, const MathStructure
 		ret = -1;
 	}
 	bool compare_with_1 = false;
-	calculate_userfunctions(mfunc, vargs[3], eo);
+	calculate_userfunctions(mfunc, vargs[2], eo);
 	MathStructure mdiff(mfunc);
 	mdiff.differentiate(vargs[2], eo);
 	if(mdiff.containsFunction(CALCULATOR->getFunctionById(FUNCTION_ID_DIFFERENTIATE))) return ret;
