@@ -131,7 +131,7 @@ int CircularShiftFunction::calculate(MathStructure &mstruct, const MathStructure
 	}
 	PrintOptions po;
 	po.base = BASE_BINARY;
-	po.base_display = BASE_DISPLAY_NONE;
+	po.base_display = BASE_DISPLAY_NORMAL;
 	po.binary_bits = bits;
 	string str = nr.print(po);
 	remove_blanks(str);
@@ -188,6 +188,12 @@ IFFunction::IFFunction() : MathFunction("if", 3, 4) {
 	setArgumentDefinition(4, new BooleanArgument());
 	setDefaultValue(4, "0");
 }
+bool IFFunction::representsScalar(const MathStructure &vargs) const {
+	return vargs.size() >= 3 && vargs[1].representsScalar() && vargs[2].representsScalar();
+}
+bool IFFunction::representsNonMatrix(const MathStructure &vargs) const {
+	return vargs.size() >= 3 && vargs[1].representsNonMatrix() && vargs[2].representsNonMatrix();
+}
 int IFFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	if(vargs[0].isNumber()) {
 		int result = vargs[0].number().getBoolean();
@@ -201,7 +207,8 @@ int IFFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, co
 		return 1;
 	}
 	mstruct = vargs[0];
-	if(mstruct.isVector()) {
+	if(mstruct.isVector() || (mstruct.isFunction() && (mstruct.function()->id() == FUNCTION_ID_HORZCAT || mstruct.function()->id() == FUNCTION_ID_VERTCAT))) {
+		MathStructure m2(vargs[1]);
 		for(size_t i = 0; i < mstruct.size(); i++) {
 			mstruct[i].eval(eo);
 			if(!mstruct[i].isNumber() && vargs[3].isZero()) {
@@ -209,10 +216,10 @@ int IFFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, co
 			}
 			int result = mstruct[i].number().getBoolean();
 			if(result > 0) {
-				if(vargs[1].isVector() && vargs[1].size() == vargs[0].size()) {
-					mstruct = vargs[1][i];
+				if((m2.isVector() || (m2.isFunction() && (m2.function()->id() == FUNCTION_ID_HORZCAT || m2.function()->id() == FUNCTION_ID_VERTCAT))) && m2.size() == vargs[0].size()) {
+					mstruct = m2[i];
 				} else {
-					mstruct = vargs[1];
+					mstruct = m2[1];
 				}
 				return 1;
 			} else if(result < 0 && vargs[3].isZero()) {
