@@ -120,6 +120,61 @@ int MergeVectorsFunction::calculate(MathStructure &mstruct, const MathStructure
 	}
 	return 1;
 }
+VertCatFunction::VertCatFunction() : MathFunction("vertcat", 1, -1) {
+	setArgumentDefinition(1, new MatrixArgument(""));
+	setArgumentDefinition(2, new MatrixArgument(""));
+}
+int VertCatFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
+	mstruct = vargs[0];
+	for(size_t i = 1; i < vargs.size(); i++) {
+		if(CALCULATOR->aborted()) return 0;
+		if(vargs[i].columns() != mstruct.columns()) {
+			CALCULATOR->error(true, _("Vertical concatenation requires equal number of columns."), NULL);
+			if(i > 1) {
+				mstruct.transform(this);
+				for(; i < vargs.size(); i++) mstruct.addChild(vargs[i]);
+				return 1;
+			}
+			return 0;
+		}
+		for(size_t i2 = 0; i2 < vargs[i].size(); i2++) {
+			mstruct.addChild(vargs[i][i2]);
+		}
+	}
+	return 1;
+}
+HorzCatFunction::HorzCatFunction() : MathFunction("horzcat", 1, -1) {
+	setArgumentDefinition(1, new MatrixArgument(""));
+	setArgumentDefinition(2, new MatrixArgument(""));
+}
+bool HorzCatFunction::representsScalar(const MathStructure &vargs) const {return false;}
+bool HorzCatFunction::representsNonMatrix(const MathStructure &vargs) const {
+	for(size_t i = 0; i < vargs.size(); i++) {
+		if(!vargs[i].representsNonMatrix()) return false;
+	}
+	return true;
+}
+int HorzCatFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
+	mstruct = vargs[0];
+	for(size_t i = 1; i < vargs.size(); i++) {
+		if(CALCULATOR->aborted()) return 0;
+		if(vargs[i].rows() != mstruct.rows()) {
+			CALCULATOR->error(true, _("Horizontal concatenation requires equal number of rows."), NULL);
+			if(i > 1) {
+				mstruct.transform(this);
+				for(; i < vargs.size(); i++) mstruct.addChild(vargs[i]);
+				return 1;
+			}
+			return 0;
+		}
+		for(size_t i2 = 0; i2 < vargs[i].size(); i2++) {
+			for(size_t i3 = 0; i3 < vargs[i][i2].size(); i3++) {
+				mstruct[i2].addChild(vargs[i][i2][i3]);
+			}
+		}
+	}
+	return 1;
+}
 MatrixToVectorFunction::MatrixToVectorFunction() : MathFunction("matrix2vector", 1) {
 	setArgumentDefinition(1, new MatrixArgument());
 }
@@ -154,30 +209,24 @@ int ColumnFunction::calculate(MathStructure &mstruct, const MathStructure &vargs
 	return 1;
 }
 RowsFunction::RowsFunction() : MathFunction("rows", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument(""));
 }
 int RowsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(!vargs[0].isMatrix()) mstruct = m_one;
-	else mstruct = (int) vargs[0].rows();
+	mstruct = (int) vargs[0].rows();
 	return 1;
 }
 ColumnsFunction::ColumnsFunction() : MathFunction("columns", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument(""));
 }
 int ColumnsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(!vargs[0].isMatrix()) mstruct = (int) vargs[0].countChildren();
-	else mstruct = (int) vargs[0].columns();
+	mstruct = (int) vargs[0].columns();
 	return 1;
 }
 ElementsFunction::ElementsFunction() : MathFunction("elements", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument(""));
 }
 int ElementsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(vargs[0].isMatrix()) {
-		mstruct = (int) (vargs[0].rows() * vargs[0].columns());
-	} else {
-		mstruct = (int) vargs[0].countChildren();
-	}
+	mstruct = (int) (vargs[0].rows() * vargs[0].columns());
 	return 1;
 }
 ElementFunction::ElementFunction() : MathFunction("element", 2, 3) {
@@ -186,37 +235,49 @@ ElementFunction::ElementFunction() : MathFunction("element", 2, 3) {
 	setArgumentDefinition(3, new IntegerArgument("", ARGUMENT_MIN_MAX_NONE, true, true, INTEGER_TYPE_SIZE));
 	setDefaultValue(3, "0");
 }
+bool ElementFunction::representsScalar(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive()) {
+		if(vargs.size() == 2 || vargs[2].isZero()) {
+			if(vargs[1].number() <= vargs[0].size()) return vargs[0][vargs[1].number().uintValue() - 1].representsScalar();
+		} else if(vargs[0].isMatrix() && vargs[1].number() <= vargs[0].size() && vargs[2].isInteger() && vargs[2].number().isPositive() && vargs[2].number() <= vargs[0][0].size()) {
+			return vargs[0][vargs[1].number().uintValue() - 1][vargs[2].number().uintValue() - 1].representsScalar();
+		}
+	}
+	return false;
+}
+bool ElementFunction::representsNonMatrix(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive()) {
+		if(vargs.size() == 2 || vargs[2].isZero()) {
+			if(vargs[1].number() <= vargs[0].size()) return vargs[0][vargs[1].number().uintValue() - 1].representsNonMatrix();
+		} else if(vargs[0].isMatrix() && vargs[1].number() <= vargs[0].size() && vargs[2].isInteger() && vargs[2].number().isPositive() && vargs[2].number() <= vargs[0][0].size()) {
+			return vargs[0][vargs[1].number().uintValue() - 1][vargs[2].number().uintValue() - 1].representsNonMatrix();
+		}
+	}
+	return false;
+}
 int ElementFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(vargs[2].number().isPositive() && vargs[0].isMatrix()) {
-		size_t row = (size_t) vargs[1].number().uintValue();
-		size_t col = (size_t) vargs[2].number().uintValue();
-		bool b = true;
-		if(col > vargs[0].columns()) {
-			CALCULATOR->error(true, _("Column %s does not exist in matrix."), format_and_print(vargs[2]).c_str(), NULL);
-			b = false;
-		}
-		if(row > vargs[0].rows()) {
-			CALCULATOR->error(true, _("Row %s does not exist in matrix."), format_and_print(vargs[1]).c_str(), NULL);
-			b = false;
-		}
-		if(b) {
-			const MathStructure *em = vargs[0].getElement(row, col);
-			if(em) mstruct = *em;
-			else b = false;
-		}
-		return b;
-	} else {
-		if(vargs[2].number().isGreaterThan(1)) {
-			CALCULATOR->error(false, _("Argument 3, %s, is ignored for vectors."), getArgumentDefinition(3)->name().c_str(), NULL);
-		}
-		size_t row = (size_t) vargs[1].number().uintValue();
-		if(row > vargs[0].countChildren()) {
-			CALCULATOR->error(true, _("Element %s does not exist in vector."), format_and_print(vargs[1]).c_str(), NULL);
+	size_t row = (size_t) vargs[1].number().uintValue();
+	size_t col = (size_t) vargs[2].number().uintValue();
+	if(col == 0) {
+		if(row > vargs[0].size()) {
+			CALCULATOR->error(true, _("Element %s does not exist in vector."), format_and_print(vargs[0]).c_str(), NULL);
 			return 0;
 		}
-		mstruct = *vargs[0].getChild(row);
+		mstruct = vargs[0][row - 1];
 		return 1;
 	}
+	if(col > vargs[0].columns()) {
+		CALCULATOR->error(true, _("Column %s does not exist in matrix."), format_and_print(vargs[1]).c_str(), NULL);
+		return 0;
+	}
+	if(row > vargs[0].rows()) {
+		CALCULATOR->error(true, _("Row %s does not exist in matrix."), format_and_print(vargs[0]).c_str(), NULL);
+		return 0;
+	}
+	const MathStructure *em = vargs[0].getElement(row, col);
+	if(em) mstruct = *em;
+	else return 0;
+	return 1;
 }
 DimensionFunction::DimensionFunction() : MathFunction("dimension", 1) {
 	setArgumentDefinition(1, new VectorArgument(""));
@@ -229,6 +290,18 @@ ComponentFunction::ComponentFunction() : MathFunction("component", 2) {
 	setArgumentDefinition(1, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 	setArgumentDefinition(2, new VectorArgument(""));
 }
+bool ComponentFunction::representsScalar(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive() && vargs[1].number() <= vargs[0].size()) {
+		return vargs[0][vargs[1].number().uintValue() - 1].representsScalar();
+	}
+	return false;
+}
+bool ComponentFunction::representsNonMatrix(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive() && vargs[1].number() <= vargs[0].size()) {
+		return vargs[0][vargs[1].number().uintValue() - 1].representsNonMatrix();
+	}
+	return false;
+}
 int ComponentFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	size_t i = (size_t) vargs[0].number().uintValue();
 	if(i > vargs[1].countChildren()) {
@@ -315,7 +388,9 @@ int PermanentFunction::calculate(MathStructure &mstruct, const MathStructure &va
 	return !mstruct.isUndefined();
 }
 CofactorFunction::CofactorFunction() : MathFunction("cofactor", 3) {
-	setArgumentDefinition(1, new MatrixArgument());
+	MatrixArgument *marg = new MatrixArgument();
+	marg->setSquareDemanded(true);
+	setArgumentDefinition(1, marg);
 	setArgumentDefinition(2, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 	setArgumentDefinition(3, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 }
@@ -333,13 +408,27 @@ int AdjointFunction::calculate(MathStructure &mstruct, const MathStructure &varg
 	if(!mstruct.adjointMatrix(eo)) return 0;
 	return !mstruct.isUndefined();
 }
-InverseFunction::InverseFunction() : MathFunction("inverse", 1) {
+InverseFunction::InverseFunction() : MathFunction("inv", 1) {
 	MatrixArgument *marg = new MatrixArgument();
+	marg->setTests(false);
 	marg->setSquareDemanded(true);
 	setArgumentDefinition(1, marg);
 }
 int InverseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	mstruct = vargs[0];
+	if(mstruct.representsScalar()) {mstruct.inverse(); return 1;}
+	if(!mstruct.isMatrix()) {
+		mstruct.eval(eo);
+		if(mstruct.representsScalar()) {mstruct.inverse(); return 1;}
+		else if(!mstruct.isVector()) return -1;
+	}
+	if(!mstruct.isMatrix() || !mstruct.matrixIsSquare()) {
+		Argument *arg = getArgumentDefinition(1);
+		arg->setTests(true);
+		arg->test(mstruct, 1, this, eo);
+		arg->setTests(false);
+		return -1;
+	}
 	return mstruct.invertMatrix(eo);
 }
 MagnitudeFunction::MagnitudeFunction() : MathFunction("magnitude", 1) {
@@ -419,35 +508,6 @@ int NormFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	}
 	return -1;
 }
-HadamardFunction::HadamardFunction() : MathFunction("hadamard", 1, -1) {
-	setArgumentDefinition(1, new VectorArgument());
-	setArgumentDefinition(2, new VectorArgument());
-}
-int HadamardFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	bool b_matrix = vargs[0].isMatrix();
-	for(size_t i3 = 1; i3 < vargs.size(); i3++) {
-		if(b_matrix) {
-			if(!vargs[i3].isMatrix() || vargs[i3].columns() != vargs[0].columns() || vargs[i3].rows() != vargs[0].rows()) {
-				CALCULATOR->error(true, _("%s() requires that all matrices/vectors have the same dimensions."), preferredDisplayName().name.c_str(), NULL);
-				return 0;
-			}
-		} else if(vargs[i3].size() != vargs[0].size()) {
-			CALCULATOR->error(true, _("%s() requires that all matrices/vectors have the same dimensions."), preferredDisplayName().name.c_str(), NULL);
-			return 0;
-		}
-	}
-	mstruct = vargs[0];
-	for(size_t i = 0; i < mstruct.size(); i++) {
-		if(b_matrix) {
-			for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
-				for(size_t i3 = 1; i3 < vargs.size(); i3++) mstruct[i][i2] *= vargs[i3][i][i2];
-			}
-		} else {
-			for(size_t i3 = 1; i3 < vargs.size(); i3++) mstruct[i] *= vargs[i3][i];
-		}
-	}
-	return 1;
-}
 bool matrix_to_rref(MathStructure &m, const EvaluationOptions &eo) {
 	size_t rows = m.rows();
 	size_t cols = m.columns();
@@ -611,29 +671,38 @@ int DotProductFunction::calculate(MathStructure &mstruct, const MathStructure &v
 	return -3;
 }
 
-EntrywiseMultiplicationFunction::EntrywiseMultiplicationFunction() : MathFunction("times", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
-}
+EntrywiseMultiplicationFunction::EntrywiseMultiplicationFunction() : MathFunction("times", 1, -1) {}
 int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
-	if(vargs[0].size() == 0) {mstruct.clear(); return 1;}
-	if(vargs[0].size() == 1) {mstruct = vargs[0][0]; return 1;}
-	if(vargs[0].size() > 2) {
-		mstruct = vargs[0][0];
+	if(vargs.size() == 0) {mstruct.clear(); return 1;}
+	if(vargs.size() == 1) {
+		mstruct = vargs[0];
+		if(!mstruct.isVector()) {
+			if(mstruct.representsScalar()) return 1;
+			mstruct.eval(eo);
+			mstruct = vargs[0];
+			if(!mstruct.isVector()) return 1;
+		}
+		mstruct.setType(STRUCT_FUNCTION);
+		mstruct.setFunction(this);
+		return 1;
+	}
+	if(vargs.size() > 2) {
+		mstruct = vargs[0];
 		bool b_scalar = mstruct.representsScalar();
-		for(size_t index = 1; index < vargs[0].size(); index++) {
+		for(size_t index = 1; index < vargs.size(); index++) {
 			if(CALCULATOR->aborted()) return 0;
-			if(b_scalar || vargs[0][index].representsScalar()) {
-				mstruct.multiply(vargs[0][index], true);
+			if(b_scalar || vargs[index].representsScalar()) {
+				mstruct.multiply(vargs[index], true);
 				if(b_scalar) b_scalar = mstruct.representsScalar();
 			} else {
-				mstruct.transform(STRUCT_VECTOR, vargs[0][index]);
+				mstruct.transform(STRUCT_VECTOR, vargs[index]);
 				mstruct.transform(this);
 			}
 		}
 		return 1;
 	}
-	mstruct = vargs[0][0];
-	MathStructure m2(vargs[0][1]);
+	mstruct = vargs[0];
+	MathStructure m2(vargs[1]);
 	bool b_eval = false;
 	if(mstruct.representsScalar() || m2.representsScalar()) {
 		mstruct *= m2;
@@ -1067,6 +1136,7 @@ int GenerateVectorFunction::calculate(MathStructure &mstruct, const MathStructur
 	return 1;
 }
 SelectFunction::SelectFunction() : MathFunction("select", 2, 4) {
+	setArgumentDefinition(1, new VectorArgument());
 	setArgumentDefinition(3, new SymbolicArgument());
 	setDefaultValue(3, "undefined");
 	setArgumentDefinition(4, new BooleanArgument());
@@ -1075,31 +1145,14 @@ SelectFunction::SelectFunction() : MathFunction("select", 2, 4) {
 int SelectFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	MathStructure mtest;
 	mstruct = vargs[0];
-	mstruct.eval(eo);
-	if(!mstruct.isVector()) {
-		mtest = vargs[1];
-		mtest.replace(vargs[2], mstruct);
-		mtest.eval(eo);
-		if(!mtest.isNumber() || mtest.number().getBoolean() < 0) {
-			CALCULATOR->error(true, _("Comparison failed."), NULL);
-			return -1;
-		}
-		if(mtest.number().getBoolean() == 0) {
-			if(vargs[3].number().getBoolean() > 0) {
-				CALCULATOR->error(true, _("No matching item found."), NULL);
-				return -1;
-			}
-			mstruct.clearVector();
-		}
-		return 1;
-	}
 	for(size_t i = 0; i < mstruct.size();) {
+		if(CALCULATOR->aborted()) return 0;
 		mtest = vargs[1];
 		mtest.replace(vargs[2], mstruct[i]);
 		mtest.eval(eo);
 		if(!mtest.isNumber() || mtest.number().getBoolean() < 0) {
 			CALCULATOR->error(true, _("Comparison failed."), NULL);
-			return -1;
+			return 0;
 		}
 		if(mtest.number().getBoolean() == 0) {
 			if(vargs[3].number().getBoolean() == 0) {
@@ -1116,7 +1169,7 @@ int SelectFunction::calculate(MathStructure &mstruct, const MathStructure &vargs
 	}
 	if(vargs[3].number().getBoolean() > 0) {
 		CALCULATOR->error(true, _("No matching item found."), NULL);
-		return -1;
+		return 0;
 	}
 	return 1;
 }
@@ -1141,7 +1194,7 @@ int LoadFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	return 1;
 }
 ExportFunction::ExportFunction() : MathFunction("export", 2, 3) {
-	setArgumentDefinition(1, new VectorArgument());
+	setArgumentDefinition(1, new MatrixArgument());
 	setArgumentDefinition(2, new FileArgument());
 	setArgumentDefinition(3, new TextArgument());
 	setDefaultValue(3, "\",\"");
