@@ -1263,7 +1263,11 @@ void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 					if(i2 > 0 && new_formula[i2 - 1] == '\\') {
 						i2++;
 					} else {
-						new_formula.replace(i2, 2, svar);
+						if(i2 + 4 < new_formula.length() && new_formula[i2 + 2] == ID_WRAP_LEFT_CH && (i3 = new_formula.find(ID_WRAP_RIGHT_CH, i2 + 3)) != string::npos) {
+							new_formula.replace(i2, i3 - i2 + 1, svar);
+						} else {
+							new_formula.replace(i2, 2, svar);
+						}
 					}
 				}
 				for(size_t sub_i = 0; sub_i < priv->v_subs_calc.size(); sub_i++) {
@@ -1538,6 +1542,13 @@ MathStructure Argument::parse(const string &str, const ParseOptions &po) const {
 }
 void Argument::parse(MathStructure *mstruct, const string &str, const ParseOptions &po) const {
 	if(b_text) {
+		MathFunction *f_cat = CALCULATOR->getFunctionById(FUNCTION_ID_CONCATENATE);
+		for(size_t i = 1; i <= f_cat->countNames(); i++) {
+			if(str.find(f_cat->getName(i).name) != string::npos) {
+				CALCULATOR->parse(mstruct, str, po);
+				return;
+			}
+		}
 		size_t pars = 0;
 		while(true) {
 			size_t pars2 = 1;
@@ -2142,13 +2153,27 @@ VectorArgument::~VectorArgument() {
 	}
 }
 bool VectorArgument::subtest(MathStructure &value, const EvaluationOptions &eo) const {
-	//if(!value.isVector()) {
+	bool subvector = false;
+	for(size_t i = 0; i < subargs.size(); i++) {
+		if(subargs[i] && (subargs[i]->type() == ARGUMENT_TYPE_VECTOR || subargs[i]->type() == ARGUMENT_TYPE_MATRIX)) {
+			subvector = true;
+			break;
+		}
+	}
+	if(subvector && value.isFunction() && (value.function()->id() == FUNCTION_ID_HORZCAT || value.function()->id() == FUNCTION_ID_VERTCAT)) {
+		value.setType(STRUCT_VECTOR);
+	}
+	if(!subvector || !value.isVector()) {
 		value.eval(eo);
-	//}
+	}
 	if(!value.isVector()) {
 		if(isLastArgument()) value.transform(STRUCT_VECTOR);
+		else if(value.representsScalar()) value.transform(STRUCT_VECTOR);
 		else return false;
 	}
+	if(value.isMatrix() && value.columns() == 1 && value.rows() > 1) {
+		value.transposeMatrix();
+	}
 	if(b_argloop && subargs.size() > 0) {
 		for(size_t i = 0; i < value.countChildren(); i++) {
 			if(!subargs[i % subargs.size()]->test(value[i], 1, NULL, eo)) {
@@ -2219,9 +2244,18 @@ MatrixArgument::MatrixArgument(const MatrixArgument *arg) {
 MatrixArgument::~MatrixArgument() {}
 bool MatrixArgument::subtest(MathStructure &value, const EvaluationOptions &eo) const {
 	value.eval(eo);
-	if(!b_square && !value.isMatrix() && value.isVector() && value.size() > 0 && !value[0].isVector()) {
-		for(size_t i = 0; i < value.size(); i++) {
-			value[i].transform(STRUCT_VECTOR);
+	if(!value.isMatrix()) {
+		if(value.isVector() && (value.size() == 0 || value[0].representsScalar())) {
+			if(CALCULATOR->usesMatlabStyleMatrices() || value.size() == 0) {
+				value.transform(STRUCT_VECTOR);
+			} else {
+				for(size_t i = 0; i < value.size(); i++) {
+					value[i].transform(STRUCT_VECTOR);
+				}
+			}
+		} else if(value.representsScalar()) {
+			value.transform(STRUCT_VECTOR);
+			value.transform(STRUCT_VECTOR);
 		}
 	}
 	return value.isMatrix() && (!b_square || value.matrixIsSquare());
