@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
+    Copyright (C) 2003-2007, 2008, 2016-2021  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@
 #include <unistd.h>
 #include <stdint.h>
 
-#define QALCULATE_MAJOR_VERSION (3)
-#define QALCULATE_MINOR_VERSION (13)
+#define QALCULATE_MAJOR_VERSION (4)
+#define QALCULATE_MINOR_VERSION (0)
 #define QALCULATE_MICRO_VERSION (0)
 
 static std::string empty_string;
@@ -82,8 +82,8 @@ typedef enum {
 #define COMPARISON_IS_EQUAL_OR_GREATER(i)	(i == COMPARISON_RESULT_EQUAL || i == COMPARISON_RESULT_GREATER || i == COMPARISON_RESULT_EQUAL_OR_GREATER)
 #define COMPARISON_IS_EQUAL_OR_LESS(i)		(i == COMPARISON_RESULT_EQUAL || i == COMPARISON_RESULT_LESS || i == COMPARISON_RESULT_EQUAL_OR_LESS)
 #define COMPARISON_IS_NOT_EQUAL(i)		(i == COMPARISON_RESULT_NOT_EQUAL || i == COMPARISON_RESULT_LESS || i == COMPARISON_RESULT_GREATER)
-#define COMPARISON_MIGHT_BE_EQUAL(i)		(i >= COMPARISON_RESULT_UNKNOWN || i == COMPARISON_RESULT_EQUAL_OR_LESS || i == COMPARISON_RESULT_EQUAL_OR_GREATER)
-#define COMPARISON_MIGHT_BE_NOT_EQUAL(i)	(i >= COMPARISON_RESULT_UNKNOWN || i == COMPARISON_RESULT_EQUAL_OR_LESS || i == COMPARISON_RESULT_EQUAL_OR_GREATER)
+#define COMPARISON_MIGHT_BE_EQUAL(i)		(i >= COMPARISON_RESULT_UNKNOWN || i == COMPARISON_RESULT_EQUAL_OR_LESS || i == COMPARISON_RESULT_EQUAL_OR_GREATER || i == COMPARISON_RESULT_EQUAL)
+#define COMPARISON_MIGHT_BE_NOT_EQUAL(i)	(i >= COMPARISON_RESULT_UNKNOWN || i == COMPARISON_RESULT_EQUAL_OR_LESS || i == COMPARISON_RESULT_EQUAL_OR_GREATER || i == COMPARISON_RESULT_NOT_EQUAL)
 
 #define NR_OF_PRIMES 600
 
@@ -308,6 +308,12 @@ typedef enum {
 #define BASE_DUODECIMAL		12
 #define BASE_HEXADECIMAL	16
 #define BASE_SEXAGESIMAL	60
+#define BASE_SEXAGESIMAL_2	62
+#define BASE_SEXAGESIMAL_3	63
+#define BASE_LATITUDE		70
+#define BASE_LATITUDE_2		71
+#define BASE_LONGITUDE		72
+#define BASE_LONGITUDE_2	73
 #define BASE_CUSTOM		-3
 #define BASE_UNICODE		-4
 #define BASE_GOLDEN_RATIO	-5
@@ -322,6 +328,8 @@ typedef enum {
 #define BASE_FP128		-33
 #define BASE_FP80		-34
 
+#define BASE_IS_SEXAGESIMAL(x) ((x >= BASE_SEXAGESIMAL && x <= BASE_SEXAGESIMAL_3) || (x >= BASE_LATITUDE && x <= BASE_LONGITUDE_2))
+
 #define EXP_BASE_3		-3
 #define EXP_PRECISION		-1
 #define EXP_NONE		0
@@ -364,7 +372,8 @@ typedef enum {
 typedef enum {
 	BASE_DISPLAY_NONE,
 	BASE_DISPLAY_NORMAL,
-	BASE_DISPLAY_ALTERNATIVE
+	BASE_DISPLAY_ALTERNATIVE,
+	BASE_DISPLAY_SUFFIX
 } BaseDisplay;
 
 typedef enum {
@@ -398,7 +407,7 @@ static const struct PrintOptions {
 	int min_exp;
 	/// Number base for displaying numbers. Default: 10
 	int base;
-	/// How prefixes for numbers in non-decimal bases will be displayed
+	/// How prefixes for numbers in non-decimal bases will be displayed. Default: BASE_DISPLAY_NONE
 	BaseDisplay base_display;
 	/// Use lower case for non-numeric characters for bases > 10. Default: false
 	bool lower_case_numbers;
@@ -444,7 +453,7 @@ static const struct PrintOptions {
 	bool spacious;
 	/// Use parentheses even when not necessary. Default: false
 	bool excessive_parenthesis;
-	/// Transform raised to 1/2 to square root function. Default: true
+	/// Transform raised to 1/2 to square root and raised to 1/3 to cbrt function. Default: true
 	bool halfexp_to_sqrt;
 	/// Minimum number of decimals to display for numbers. Default: 0
 	int min_decimals;
@@ -482,7 +491,7 @@ static const struct PrintOptions {
 	bool spell_out_logical_operators;
 	/// Displays children of the structure with no higher precision than the parent. Default: true
 	bool restrict_to_parent_precision;
-	/// Restrict the length of numerators and demonitor as integers in decimal mode for fractional display of numbers. Default: false
+	/// Restrict the length of numerators and denominator as integers in decimal mode for fractional display of numbers. Default: false
 	bool restrict_fraction_length;
 	/// Transform exponentiation positive base and unit fraction exponent (if denominator < 10) to root function. Default: false
 	bool exp_to_root;
@@ -554,7 +563,7 @@ typedef enum {
 	POST_CONVERSION_OPTIMAL_SI,
 	/// Convert to base units
 	POST_CONVERSION_BASE,
-	/// Convert to the the least amount of units. Non-SI units is kept (if optimal), but for conversion only SI units are used.
+	/// Convert to the least amount of units. Non-SI units is kept (if optimal), but for conversion only SI units are used.
 	POST_CONVERSION_OPTIMAL
 } AutoPostConversion;
 
@@ -595,9 +604,14 @@ typedef enum {
 	/// In the "parse implicit multiplication first" mode, implicit multiplication is parsed before explicit multiplication (<quote>12/2(1+2) = 12/(2*3) = 2</quote>, <quote>5x/5y = (5*x)/(5*y) = x/y</quote>).
 	PARSING_MODE_IMPLICIT_MULTIPLICATION_FIRST,
 	/// In the conventional mode implicit multiplication does not differ from explicit multiplication (<quote>12/2(1+2) = 12/2*3 = 18</quote>, <quote>5x/5y = 5*x/5*y = xy</quote>).
-	PARSING_MODE_CONVENTIONAL
+	PARSING_MODE_CONVENTIONAL,
+	// as immediate execution mode in simple traditional calculators (e.g. 5+2*3=(5+2)*3=21)
+	PARSING_MODE_CHAIN,
+	PARSING_MODE_RPN
 } ParsingMode;
 
+#define PARSE_PERCENT_AS_ORDINARY_CONSTANT 0x10
+
 typedef enum {
 	/// Ignores uncertainties and uses the middle value of intervals
 	INTERVAL_CALCULATION_NONE,
@@ -617,13 +631,13 @@ static const struct ParseOptions {
 	bool unknowns_enabled;
 	/// If units will be parsed. Default: true
 	bool units_enabled;
-	/// If Reverse Polish Notation syntax will be used. Default: false
+	/// If Reverse Polish Notation syntax will be used. Default: false (deprecated, use parsing_mode = PARSING_MODE_RPN instead)
 	bool rpn;
 	/// Base of parsed numbers. Default: 10
 	int base;
 	/// When implicit multiplication is limited variables, functions and units must be separated by a space, operator or parenthesis ("xy" does not equal "x * y").  Default: false
 	/**
-	* If the limit implicit multiplication mode is activated, the use of implicite multiplication when parsing expressions and displaying results will be limited to avoid confusion. For example, if this mode is not activated and "integrte(5x)" is accidently typed instead of "integrate(5x)", the expression is interpreted as "int(e * e * (5 * x) * gr * t)". If limit implicit multiplication is turned on to mistyped expression would instead show an error telling that "integrte" is not a valid variable, function or unit (unless unknowns is not enabled in which case the result will be "5 'integrate' * x".
+	* If the limit implicit multiplication mode is activated, the use of implicit multiplication when parsing expressions and displaying results will be limited to avoid confusion. For example, if this mode is not activated and "integrte(5x)" is accidently typed instead of "integrate(5x)", the expression is interpreted as "int(e * e * (5 * x) * gr * t)". If limit implicit multiplication is turned on to mistyped expression would instead show an error telling that "integrte" is not a valid variable, function or unit (unless unknowns is not enabled in which case the result will be "5 'integrate' * x".
 	*/
 	bool limit_implicit_multiplication;
 	/// If and when precisions will be read from number of digits in a number. Default: DONT_READ_PRECISION
@@ -640,7 +654,7 @@ static const struct ParseOptions {
 	MathStructure *unended_function;
 	/// Preserve the expression structure as much as possible. Default: false
 	bool preserve_format;
-	/// Default dataset. Used for object.property syntax without a preceeding data set. Default: NULL
+	/// Default dataset. Used for object.property syntax without a preceding data set. Default: NULL
 	DataSet *default_dataset;
 	/// Parsing mode. Default: PARSING_MODE_ADAPTIVE
 	ParsingMode parsing_mode;
@@ -659,7 +673,7 @@ static const struct EvaluationOptions {
 	ApproximationMode approximation;
 	/// If units will be synced/converted to allow evaluation (ex. 1 min + 1 s=60 s+ 1 s = 61 s). Default: true
 	bool sync_units;
-	/// If units with complex/non-linear relations (ex. degress celsius and fahrenheit) will synced/converted. Default: true
+	/// If units with complex/non-linear relations (ex. degrees Celsius and Fahrenheit) will synced/converted. Default: true
 	bool sync_nonlinear_unit_relations;
 	/// If unit prefixes in original expression will be kept. Default: false
 	bool keep_prefixes;
@@ -669,7 +683,7 @@ static const struct EvaluationOptions {
 	bool calculate_functions;
 	/// If comparisons will be evaluated (ex. 5>2 => 1). Default: true
 	int test_comparisons;
-	/// If a varaible will be isolated to the left side in equations/comparisons (ex. x+y=2 => x=2-y). Default: true
+	/// If a variable will be isolated to the left side in equations/comparisons (ex. x+y=2 => x=2-y). Default: true
 	bool isolate_x;
 	/// If factors (and bases) containing addition will be expanded (ex. z(x+y)=zx+zy). Default: true
 	int expand;
@@ -773,6 +787,8 @@ extern Calculator *calculator;
 #define SIGN_INFINITY			"∞"
 #define SIGN_PLUSMINUS			"±"
 
+#define THIN_SPACE " "
+
 #define ID_WRAP_LEFT_CH		'{'
 #define ID_WRAP_RIGHT_CH	'}'
 
